# 
# remote control interface for the Agilent Infiniium Oscilloscope

set frame "Scope"
set frametitle "Agilent Infiniium Oscilloscope"
set icontitle "Scope"

if {[info exists env(MIDASScopeSpectrumServer)] == 0} then {set SpectrumServer MemSas} \
    else {set SpectrumServer $env(MIDASScopeSpectrumServer)}
    
if {[info exists env(MIDASScopeRegisterServer)] == 0} then {set RegisterServer Scope} \
    else {set RegisterServer $env(MIDASScopeRegisterServer)}


set MaxTries 20

set CurrentChannel 1
set WaveFormSize 4096

set IPaddress "10.0.0.5"

proc define-register {class name attributes} {
   global RegisterServer

   set register /$RegisterServer/$name
   EG "register :pattern '$register' !delete"
   EG "register :name '$register' :class '$class' !new"
   EG "register :name '$register' :object '$attributes'"
}

proc wait_for_completion {args} {
    global RegisterServer MaxTries

    set tries 1

    while {$tries < $MaxTries} {

        if {[llength $args] == 0} then {
            set-footer "Waiting for action to complete ($tries)"
        } else {
            set-footer "[first $args] ($tries)"
        }

        set z [catch {EG "register :name '/$RegisterServer/Action' ?value"} m ]

        if {$z != 0} then {
            midas-warning "Unable to confirm state of action\n$m"
            clear-footer
            return 1
        }

        if {$m == 0} then {
            clear-footer
            return 0
        }
        incr tries
        after 500
    }

    midas-warning "Action has not completed\nGiving up waiting"
    clear-footer
    return 1
}

proc Create_Scope {} {
   global SpectrumServer WaveFormSize
   
    define-register generic Response "-nScope_Response -tstring"
    define-register generic Request  "-nScope_Request -tstring"
    define-register sigTask  Sig1    "-tScopeSigUsr1 -sUSR1"
    define-register generic Action   "-nScope_Action -tint"
    define-register generic Trace    "-nScope_Trace -tint"
    
    UnMap_Scope
    delete_spectra
    create_spectra
}

proc create_spectra {} {
    global SpectrumServer WaveFormSize

    for {set c 1} {$c <= 4} {incr c} {
        set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $WaveFormSize :name 'Scope.WaveForm$c' !new)" } m]
    }
}

proc delete_spectra {} {
    global SpectrumServer

    for {set c 1} {$c <= 4} {incr c} {
        set z [catch {eg "(spectrum :path '/$SpectrumServer'  :name 'Scope.WaveForm$c' !delete)" } m]
    }
}

proc Open_Scope {} {
   global IPaddress
   
    write-register Request $IPaddress
    write-register Action 1
    write-register Sig1 @0
    if {[wait_for_completion "Opening Scope Connection"] == 0} then {
        insert "[read-register Response]"
    }
}

proc Close_Scope {} {
    write-register Action 2
    write-register Sig1 @0
    if {[wait_for_completion "Closing Scope Connection"] == 0} then {
        insert "[read-register Response]"
    }
}

proc Query_Scope {command} {
    write-register Request $command
    write-register Action 5
    write-register Sig1 @0
    if {[wait_for_completion "Scope query"] == 0} then {
        set response [read-register Response]
        insert "$response"
        return "$response"
    } else {
        return "FAILED"
    }
}

proc Read_Scope {} {
    write-register Action 4
    write-register Sig1 @0
    if {[wait_for_completion "Scope read"] == 0} then {
        set response [read-register Response]
        insert "$response"
        return "$response"
    } else {
        return "FAILED"
    }
}

proc ReadBytes_Scope {length} {
    write-register Request "$length\n"
    write-register Action 6
    write-register Sig1 @0
    if {[wait_for_completion "Scope read"] == 0} then {
        set response [read-register Response]
        insert "$response"
        return "$response"
    } else {
        return "FAILED"
    }
}

proc Command_Scope {command} {
    write-register Request $command
    write-register Action 3
    write-register Sig1 @0
    if {[wait_for_completion "Scope command"] == 0} then {
        insert "[read-register Response]"
    }
}

proc Read_WaveForm {c} {
    write-register Request "$c\n"
    write-register Action 7
    write-register Sig1 @0
    wait_for_completion "Waveform read"
}

proc Map_Scope {} {
    write-register Action 10
    write-register Sig1 @0
    wait_for_completion "Mapping spectra"
}


proc UnMap_Scope {} {
    write-register Action 11
    write-register Sig1 @0
    wait_for_completion "Unmapping spectra"
}

proc scope%initialise {} {
    global CurrentChannel WaveFormSize

    Command_Scope *RST
    Command_Scope *CLS
    Command_Scope ":SYSTem:HEADer OFF"
    Command_Scope ":TIMebase:REFerence CENTer;RANge 2e-3;POSition 20e-6"
    Command_Scope ":CHANnel[set CurrentChannel]:RANGe 1.6;OFFSet -400e-3"
    Command_Scope ":TRIGger:EDGE:SOURce CHANnel[set CurrentChannel];SLOPe POSitive"
    Command_Scope ":TRIGger:LEVel CHANnel[set CurrentChannel],-0.40"
    Command_Scope ":ACQuire:MODE RTIMe;AVERage OFF;POINTs $WaveFormSize"
}

proc scope%acquire_data {} {
    global CurrentChannel

    Command_Scope ":DIGitize CHANnel$CurrentChannel"
    Command_Scope ":CHANnel[set CurrentChannel]:DISPlay ON"
}

proc scope%auto_measurements {} {
    global CurrentChannel

    Command_Scope ":MEASure:SENDvalid ON"
    set response [Query_Scope ":MEASure:VPP? CHANnel$CurrentChannel"]
    set flag [string range $response [expr [string length $response]-1] end]
    set result [string range $response 0 [expr [string length $response]-2]]
    if {$flag != "0"} {
        insert "Automated vpp measurement error with result $flag"
    } else {
        insert "Volts peak-to-peak is $result"
    }
    set response [Query_Scope ":MEASure:FREQuency? CHANnel$CurrentChannel"]
    set flag [string range $response [expr [string length $response]-1] end]
    set result [string range $response 0 [expr [string length $response]-2]]
    if {$flag != "0"} {
        insert "Automated frequency measurement error with result $flag"
    } else {
        insert "Frequency is $result"
    }
}

proc scope%transfer_data {} {
    global CurrentChannel

    Command_Scope ":WAVeform:SOURce CHANnel$CurrentChannel"
    Command_Scope ":WAVeform:FORMat WORD"
    Command_Scope ":WAVeform:BYTeorder LSBFirst"
    
    set xinc [Query_Scope ":WAVeform:XINCrement?"]
    set xorg [Query_Scope ":WAVeform:XORigin?"]
    set xref [Query_Scope ":WAVeform:XREFerence?"]
    set yinc [Query_Scope ":WAVeform:YINCrement?"]
    set yorg [Query_Scope ":WAVeform:YORigin?"]
    set yref [Query_Scope ":WAVeform:YREFerence?"]
}

proc scope%read_waveform {} {
    global CurrentChannel SpectrumServer
    Read_WaveForm $CurrentChannel
    inform sd show /$SpectrumServer/Scope.WaveForm$CurrentChannel
}

proc scope%setup {} {
   scope%initialise
   scope%acquire_data
   scope%auto_measurements
   scope%transfer_data
   UnMap_Scope
   Map_Scope
}


proc click-ipaddr {vl} {global IPaddress; set IPaddress $vl}

proc click-command {s} {Command_Scope "$s"}
proc click-query   {s} {Query_Scope "$s"}

proc click_channel {vl} {global CurrentChannel; set CurrentChannel $vl}

proc click-trace {vl} {
    write-register Trace $vl
    :modify TRACE :value [read-register Trace]
}

proc make-frame {} {
  global env IPaddress CurrentChannel
  
set rows 10
set cols 90

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-name "[self icon-title]"

   :panel P1 :layout T :padx 10 :pady 10
      :layout H; :row-gap 10; :col-gap 10

      :text IPADDR       :label "Scope IP Address:"   :value $IPaddress :w 16
      :button CONNECT    :label "Connect to Scope"     :command Open_Scope
      :button DISCONNECT :label "Disconnect"           :command Close_Scope
      :button CREATE     :label "Create Scope Session" :command Create_Scope
   
   :panel P2  :layout T :padx 10 :pady 10
       :layout H; :col-gap 10
       
       :number CHANNEL :label "Scope Channel"  :value $CurrentChannel :w 2 :min 1 :max 4
       :button SETUP   :label "Setup Scope"    :command scope%setup
       :hskip 40
       :button READ    :label "Read Waveform"  :command scope%read_waveform
       :hskip 60
       :checkbox TRACE :label "Enable Gateway Trace Output" 

   :panel P3 :layout T
       :galley LIST :rows $rows :cols $cols :deaf T  :font variable
    
   :panel P4 :layout T :pady 10
       :layout V; :row-gap 20
       :text COMMAND :label "Scope Command" :w 72  :valuex 14c
       :text QUERY   :label "Scope Query"   :w 72  :valuex 14c

   open-frame

  :galley_size LIST $rows $cols
}

proc click-configure {W} {:galley_resize LIST}

proc insert {s} {:modify LIST :strings $s\n}

proc start args {
   global started
   if {$started == 0} then {
      make-frame
      set started 1
   } else {open-frame}
}

